/*
 * Copyright (c) 2010-2011, The MiCode Open Source Community (www.micode.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.micode.notes.tool;

import android.content.Context;
import android.preference.PreferenceManager;

import net.micode.notes.R;
import net.micode.notes.ui.NotesPreferenceActivity;
/*1. 简介：字面意义是资源分析器，实际上就是获取资源并且在程序中使用，比如颜色图片等
 实现方法：主要利用R.java这个类，其中包括
 R.id      组件资源引用
 R.drawable  图片资源 （被使用）
 R.layout  布局资源
 R.menu   菜单资源
 R.String  文字资源
 R.style    主题资源 （被使用）
 在按顺序设置好相应的id后，就可以编写简单的getXXX函数获取需要的资源

 特殊的变量 ：
 @BG_DEFAULT_COLOR 默认背景颜色（黄）
 BG_DEFAULT_FONT_SIZE 默认文本大小（中）*/
public class ResourceParser {
    //以下五个为颜色常量赋值
    public static final int YELLOW           = 0;
    public static final int BLUE             = 1;
    public static final int WHITE            = 2;
    public static final int GREEN            = 3;
    public static final int RED              = 4;
    //为默认背景颜色常量赋值
    public static final int BG_DEFAULT_COLOR = YELLOW;
    //为文本大小常量赋值
    public static final int TEXT_SMALL       = 0;
    public static final int TEXT_MEDIUM      = 1;
    public static final int TEXT_LARGE       = 2;
    public static final int TEXT_SUPER       = 3;
    //为默认文字大小常量赋值
    public static final int BG_DEFAULT_FONT_SIZE = TEXT_MEDIUM;
    //便签背景资源
    public static class NoteBgResources {
        //便签编辑背景颜色数组
        private final static int [] BG_EDIT_RESOURCES = new int [] {
            R.drawable.edit_yellow,
            R.drawable.edit_blue,
            R.drawable.edit_white,
            R.drawable.edit_green,
            R.drawable.edit_red
        };
        //便签编辑标题颜色数组
        private final static int [] BG_EDIT_TITLE_RESOURCES = new int [] {
            R.drawable.edit_title_yellow,
            R.drawable.edit_title_blue,
            R.drawable.edit_title_white,
            R.drawable.edit_title_green,
            R.drawable.edit_title_red
        };
        //获取便签背景资源
        public static int getNoteBgResource(int id) {
            return BG_EDIT_RESOURCES[id];
        }
        //获取便签标题背景资源
        public static int getNoteTitleBgResource(int id) {
            return BG_EDIT_TITLE_RESOURCES[id];
        }
    }
    //如果没有要求就获取默认背景id
    public static int getDefaultBgId(Context context) {
        if (PreferenceManager.getDefaultSharedPreferences(context).getBoolean(
                NotesPreferenceActivity.PREFERENCE_SET_BG_COLOR_KEY, false)) {
            return (int) (Math.random() * NoteBgResources.BG_EDIT_RESOURCES.length);
        } else {
            return BG_DEFAULT_COLOR;
        }
    }
    //便签项目背景颜色设置
    public static class NoteItemBgResources {
        //背景上方颜色设置
        private final static int [] BG_FIRST_RESOURCES = new int [] {
            R.drawable.list_yellow_up,
            R.drawable.list_blue_up,
            R.drawable.list_white_up,
            R.drawable.list_green_up,
            R.drawable.list_red_up
        };
        //背景默认颜色设置
        private final static int [] BG_NORMAL_RESOURCES = new int [] {
            R.drawable.list_yellow_middle,
            R.drawable.list_blue_middle,
            R.drawable.list_white_middle,
            R.drawable.list_green_middle,
            R.drawable.list_red_middle
        };
        //背景下方颜色设置
        private final static int [] BG_LAST_RESOURCES = new int [] {
            R.drawable.list_yellow_down,
            R.drawable.list_blue_down,
            R.drawable.list_white_down,
            R.drawable.list_green_down,
            R.drawable.list_red_down,
        };
        //背景单一颜色设置
        private final static int [] BG_SINGLE_RESOURCES = new int [] {
            R.drawable.list_yellow_single,
            R.drawable.list_blue_single,
            R.drawable.list_white_single,
            R.drawable.list_green_single,
            R.drawable.list_red_single
        };
        //获取背景上方颜色
        public static int getNoteBgFirstRes(int id) {
            return BG_FIRST_RESOURCES[id];
        }
        //获取背景下方颜色
        public static int getNoteBgLastRes(int id) {
            return BG_LAST_RESOURCES[id];
        }
        //获取背景单一颜色
        public static int getNoteBgSingleRes(int id) {
            return BG_SINGLE_RESOURCES[id];
        }
        //获取背景默认颜色
        public static int getNoteBgNormalRes(int id) {
            return BG_NORMAL_RESOURCES[id];
        }
        //获取文件夹背景颜色
        public static int getFolderBgRes() {
            return R.drawable.list_folder;
        }
    }
    //此类设置widget（窗口）背景颜色
    public static class WidgetBgResources {
        //设置2*2窗口背景颜色
        private final static int [] BG_2X_RESOURCES = new int [] {
            R.drawable.widget_2x_yellow,
            R.drawable.widget_2x_blue,
            R.drawable.widget_2x_white,
            R.drawable.widget_2x_green,
            R.drawable.widget_2x_red,
        };
        //获取2*2窗口背景颜色
        public static int getWidget2xBgResource(int id) {
            return BG_2X_RESOURCES[id];
        }
        //设置4*4窗口背景颜色
        private final static int [] BG_4X_RESOURCES = new int [] {
            R.drawable.widget_4x_yellow,
            R.drawable.widget_4x_blue,
            R.drawable.widget_4x_white,
            R.drawable.widget_4x_green,
            R.drawable.widget_4x_red
        };
        //获取4*4窗口背景颜色
        public static int getWidget4xBgResource(int id) {
            return BG_4X_RESOURCES[id];
        }
    }
    //此类设置文本显示字体大小
    public static class TextAppearanceResources {
        //设置文本显示字体大小
        private final static int [] TEXTAPPEARANCE_RESOURCES = new int [] {
            R.style.TextAppearanceNormal,
            R.style.TextAppearanceMedium,
            R.style.TextAppearanceLarge,
            R.style.TextAppearanceSuper
        };
        //获取文本显示文字字体大小，如果id大于显示的长度则返回默认字体大小
        public static int getTexAppearanceResource(int id) {
            /**
             * HACKME: Fix bug of store the resource id in shared preference.
             * The id may larger than the length of resources, in this case,
             * return the {@link ResourceParser#BG_DEFAULT_FONT_SIZE}
             */
            if (id >= TEXTAPPEARANCE_RESOURCES.length) {
                return BG_DEFAULT_FONT_SIZE;
            }
            return TEXTAPPEARANCE_RESOURCES[id];
        }
        //获取文本显示的长度
        public static int getResourcesSize() {
            return TEXTAPPEARANCE_RESOURCES.length;
        }
    }
}
