/*
 * Copyright (c) 2010-2011, The MiCode Open Source Community (www.micode.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.micode.notes.ui;

import android.content.Context;
import android.database.Cursor;
import android.view.View;
import android.view.ViewGroup;
import android.widget.CursorAdapter;
import android.widget.LinearLayout;
import android.widget.TextView;

import net.micode.notes.R;
import net.micode.notes.data.Notes;
import net.micode.notes.data.Notes.NoteColumns;


public class FoldersListAdapter extends CursorAdapter {//CursorAdapter是Cursor和ListView的接口，FoldersListAdapter继承了CursorAdapter的类，
    // 主要作用是便签数据库和用户的交互，这里就是用folder（文件夹）的形式展现给用户。
    public static final String [] PROJECTION = {//调用数据库中便签的ID和片段
        NoteColumns.ID,//定义ID和片段
        NoteColumns.SNIPPET
    };

    public static final int ID_COLUMN   = 0;//语句：定义常量，文件夹ID栏编号
    public static final int NAME_COLUMN = 1;

    public FoldersListAdapter(Context context, Cursor c) {//FoldersListAdapter类调用了父类CursorAdpater的构造函数
        super(context, c);
        // TODO Auto-generated constructor stub
    }

    @Override
    public View newView(Context context, Cursor cursor, ViewGroup parent) {//该重载函数实现了添加一个新的文件夹列表项的功能
        return new FolderListItem(context);
    }

    @Override
    public void bindView(View view, Context context, Cursor cursor) {//绑定布局文件
        if (view instanceof FolderListItem) {//判断视图是否是文件夹列表的一项
            String folderName = (cursor.getLong(ID_COLUMN) == Notes.ID_ROOT_FOLDER) ? context
                    .getString(R.string.menu_move_parent_folder) : cursor.getString(NAME_COLUMN);
            ((FolderListItem) view).bind(folderName);//绑定文件视图
        }
    }

    public String getFolderName(Context context, int position) {//根据数据库中标签的ID得到标签的各项内容
        Cursor cursor = (Cursor) getItem(position);
        return (cursor.getLong(ID_COLUMN) == Notes.ID_ROOT_FOLDER) ? context
                .getString(R.string.menu_move_parent_folder) : cursor.getString(NAME_COLUMN);
    }

    private class FolderListItem extends LinearLayout {
        private TextView mName;

        public FolderListItem(Context context) {
            super(context);
            inflate(context, R.layout.folder_list_item, this);
            mName = (TextView) findViewById(R.id.tv_folder_name);
        }

        public void bind(String name) {
            mName.setText(name);
        }
    }

}
